/****
 * CImgWndPane.h
 *
 *	Image window edit class for the SignWriter symbol editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/
 
 
#pragma once


#include "CPane.h"
#include "CCharGrid.h"
#include "CBitMap.h"
#include "CSymWndGrid.h"




struct CImgWndPane : CPane {	


				/* Symbol code */
	short		itsSymCode;
	
	
				/* Symbol variation */
	short		itsSymVar;
	
	
				/* Fill variation */
	short		itsFillVar;
	
	
				/* Image number */
	short		itsImage;
	
	
				/* Symbol window */
	CSymWndGrid	*itsSymWind;
	
	
				/* Image bitmap */
	CBitMap		*itsBitMap;	

	
				/* Tool pane */
	CCharGrid	*itsToolPane;
	
	
				/* Current edit tool */
	short		itsCurrentTool;
	
	
				/* Current image frame */
	LongPt		itsFrame;
	
	
				/* Current center point */
	LongPt		itsCenter;
	
	
				/* Current selection */
	LongRect	selRect;
	
	
				/* Initialization */						
	void		IImgWndPane (CView *anEnclosure, CBureaucrat *aSupervisor,
							 short aWidth, short aHeight,
							 short aHEncl, short aVEncl,
							 SizingOption aHSizing, SizingOption aVSizing,
							 short symCode, short symVar, short fillVar, short image,
							 CSymWndGrid *symWind);
							
	void		SetToolPane (CCharGrid *pane);
	void		Dispose (void);
	
	
				/* Commands */
	void		DoCommand (long theCommand);
	void		UpdateMenus (void);

	void		DoCopy (void);
	void		DoPaste (void);
	void		DoClear (short taskIndex);
	
	void		DoRevertImage (void);
	
	void		DoInvertVert (void);
	void		DoInvertHorz (void);
	void		DoRotate45 (void);
	void		DoRotate90 (void);
	void		DoEnlarge50 (void);
	

				/* Actions */
	void		DoClick (Point hitPt, short modifierKeys, long when);
	void		AdjustCursor (Point where, RgnHandle mouseRgn);
	Boolean		Close (void);
	
	
				/* Conversion */
	void		BitMapToPaneRect (Rect *mapRect, Rect *paneRect);
	void		PaneToBitMapRect (Rect *paneRect, Rect *mapRect);
	
	void		LBitMapToPaneRect (LongRect *mapRect, LongRect *paneRect);
	void		LPaneToBitMapRect (LongRect *paneRect, LongRect *mapRect);
	
	void		BitMapToPanePt (LongPt *mapPt, LongPt *panePt);
	void		PaneToBitMapPt (LongPt *panePt, LongPt *mapPt);

	
				/* Draw */
	void		Draw (Rect *area);
	void		Update (BitMap *map, LongRect *area);
	void		UpdateFrame (void);
	void		UpdateCenter (void);

	
				/* Misc. */	
	void		KillSelRect (void);
	void		KillFrame (void);
	void		KillCenter (void);
	void		Deactivate (void);
	void 		Dawdle (long *maxSleep);
	
};