/****
 * CSymWndDialog.h
 *
 *	Symbol window dialog classes for the SignWriter symbol editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/


#pragma once



#include "CCheckBox.h"
#include "CDLOGDirector.h"
#include "CSymWndGrid.h"
#include "Symbols.h"





/**************************  Common dialog class  ****************************/


struct CSymWndDialog : CDLOGDirector {


				/* Image class */
	ImageClass	itsClass;
	
	
				/* # rotations */
	short		itsRots;
	
	
				/* # flops */
	short		itsFlops;
	

				/* Symbol fill variation */
	short		itsFillVar;


				/* Symbol window */
	CSymWndGrid	*itsSymWind;
	
	
			/* Initialization */
	void	ISymWndDialog (short DLOGid, CDirectorOwner *aSupervisor);
	
		
			/* Dialog */
	long	GetNewAttrs (ImageClass *imgClass, short *rots, short *flops);
	
	
			/* Check boxes */
	void    AttrsToCheckBoxes (void);
	void    CheckBoxesToAttrs (void);


			/* Validation */
	Boolean	Validate (void);

};



/**************************  Hand dialog class  ****************************/


struct CHandDialog : CSymWndDialog {


				/* Check boxes */
	CCheckBox	*symVert;
	CCheckBox	*symHorz;
	CCheckBox	*ident45;
	CCheckBox	*ident90;
	CCheckBox	*ident180;	
	
	
			/* Initialization */
	void	IHandDialog (CDirectorOwner *aSupervisor);
	
	
			/* Check boxes */
	void    AttrsToCheckBoxes (void);
	void    CheckBoxesToAttrs (void);
	
	
			/* Commands */
	void	DoCommand (long theCommand);
	
};



/**************************  Movement dialog class  **************************/


struct CMvmtDialog : CSymWndDialog {


				/* Check boxes */
	CCheckBox	*symVert;
	CCheckBox	*symHorz;
	CCheckBox	*ident45;
	CCheckBox	*ident90;
	CCheckBox	*ident180;
	CCheckBox	*vertRot;	
	
	
			/* Initialization */
	void	IMvmtDialog (CDirectorOwner *aSupervisor);
	
	
			/* Check boxes */
	void    AttrsToCheckBoxes (void);
	void    CheckBoxesToAttrs (void);
	
	
			/* Commands */
	void	DoCommand (long theCommand);
	
};



/**************************  Face dialog class  ****************************/


struct CFaceDialog : CSymWndDialog {


				/* Check boxes */
	CCheckBox	*symVert;
	CCheckBox	*express1;
	CCheckBox	*express2;
	CCheckBox	*express3;	
	CCheckBox	*express7;	
	CCheckBox	*contact7;	

	
			/* Initialization */
	void	IFaceDialog (CDirectorOwner *aSupervisor);
	
	
			/* Check boxes */
	void    AttrsToCheckBoxes (void);
	void    CheckBoxesToAttrs (void);
	
	
			/* Commands */
	void	DoCommand (long theCommand);
	
};



/**************************  Shoulder dialog class  *************************/


struct CSholDialog : CSymWndDialog {


				/* Check boxes */
	CCheckBox	*symVert;
	CCheckBox	*shol1;
	CCheckBox	*shol2;
	
	
			/* Initialization */
	void	ISholDialog (CDirectorOwner *aSupervisor);
	
	
			/* Check boxes */
	void    AttrsToCheckBoxes (void);
	void    CheckBoxesToAttrs (void);
	
	
			/* Commands */
	void	DoCommand (long theCommand);
	
};



/**************************  Contact dialog class  **************************/


struct CContDialog : CSymWndDialog {


				/* Check boxes */
	CCheckBox	*symVert;
	
	
			/* Initialization */
	void	IContDialog (CDirectorOwner *aSupervisor);
	
	
			/* Check boxes */
	void    AttrsToCheckBoxes (void);
	void    CheckBoxesToAttrs (void);
	
	
			/* Commands */
	void	DoCommand (long theCommand);
	
};



/**************************  'Other' dialog class  **************************/


struct COthrDialog : CSymWndDialog {


				/* Check boxes */
	CCheckBox	*symVert;
	CCheckBox	*line2;
	CCheckBox	*line3;
	
	
			/* Initialization */
	void	IOthrDialog (CDirectorOwner *aSupervisor);
	
	
			/* Check boxes */
	void    AttrsToCheckBoxes (void);
	void    CheckBoxesToAttrs (void);
	
	
			/* Commands */
	void	DoCommand (long theCommand);
	
};