/****
 * CFileWindow.c
 *
 *	Document methods for the SignWriter symbol editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/
 


#include "Global.h"
#include "Commands.h"
#include "CApplication.h"
#include "CBartender.h"
#include "CBureaucrat.h"
#include "CDataFile.h"
#include "CDecorator.h"
#include "CDesktop.h"
#include "CFileWindow.h"
#include "CFileWndDialog.h"
#include "CFileWndGrid.h"
#include "CFileWndBox.h"
#include "CPaneBorder.h"
#include "CPanorama.h"
#include "CProgWindow.h"
#include "CScrollPane.h"
#include "CSelector.h"
#include "CWindow.h"
#include <Packages.h>
#include "Symbols.h"
#include "SymEditConsts.h"
#include "TBUtilities.h"
#include "TCLUtilities.h"





/*** Local Constants ***/

#define SYMBUF_SIZE		64000



/*** Global Variables ***/

extern	CBureaucrat  *gGopher;		/* The gopher					 */
extern	CBartender	 *gBartender;	/* The menu handling object 	 */
extern	CDecorator	 *gDecorator;	/* Window dressing object		 */
extern	CDesktop	 *gDesktop;		/* The enclosure for all windows */
extern	OSType		  gSignature;	/* The application's signature 	 */


extern  CFileWndGrid *gFileWind;	/* File window grid			 	 */
extern	short		  gSymWinds; 	/* Symbol windows open			 */
extern  void	  	 *gClipSym;		/* Clipboard 					 */



/***
 *  IFileWindow
 *
 ***/
 
void CFileWindow::IFileWindow (CApplication *aSupervisor, Boolean printable)

{
	CDocument::IDocument(aSupervisor, printable);
	
	itsGridPane = NULL;		// panes created in BuildWindow
	itsBoxPane  = NULL;

	dirty = FALSE;			// file has not been modified
}



/***
 * Dispose
 *
 ***/

void CFileWindow::Dispose (void)

{
	inherited::Dispose();
}



/***
 * DoCommand
 *
 *	Manage file window commands.
 *
 ***/

void CFileWindow::DoCommand (long theCommand)

{
	if (theCommand < 0) {
		switch (HiShort(-theCommand)) {
		
			case cmdSelectCell:
				itsBoxPane->Update(itsGridPane->ItemToSymCode(LoShort(-theCommand)));
				gBartender->UpdateMenuBar(); 
				break;
						
			default:
				inherited::DoCommand(theCommand);
		}
		
	} else {
		switch (theCommand) {
		
					
			case cmdCut:
			
				itsGridPane->DoCopy();
				itsGridPane->DoClear(iCUT);
				break;
				
					
			case cmdCopy:
			
				itsGridPane->DoCopy();
				break;
				
				
			case cmdPaste:
			
				itsGridPane->DoPaste();
				break;
				
				
			case cmdClear:
			
				itsGridPane->DoClear(iCLEAR);
				break;
				
				
		 	case cmdInsertCell:
			
				itsGridPane->DoInsertCell();
				break;
			
			
		 	case cmdDeleteCell:
		 	
				itsGridPane->DoDeleteCell();
				break;	
			
			
			case cmdSymOperation:
			
				DoSymbolOperation();
				break;			
				
				
			default:
			
				inherited::DoCommand(theCommand);
				break;			
		}
    }
}



/***
 * UpdateMenus
 *
 *  Manage file window menus.
 *
 ***/

void CFileWindow::UpdateMenus (void)

{
  	inherited::UpdateMenus();

 	gBartender->EnableCmd(cmdAbout);
 	
	gBartender->DisableCmd(cmdNew);
	gBartender->DisableCmd(cmdOpen);
	gBartender->DisableCmd(cmdPrint);
	gBartender->DisableCmd(cmdPageSetup);
				
 	gBartender->EnableCmd(cmdClose);
 	gBartender->EnableCmd(cmdSave);
 	gBartender->EnableCmd(cmdSaveAs);
 	
 	
 	if (itsFile != NULL)
 		gBartender->EnableCmd(cmdRevert);
 	

	if ((gGopher == this) && (itsGridPane->GetSelection() != SELnone)) {
		
		gBartender->EnableCmd(cmdCut);
	 	gBartender->EnableCmd(cmdCopy);
	 	gBartender->EnableCmd(cmdClear);
	 	gBartender->EnableCmd(cmdDeleteCell);
	 	
	 	if (SymEmpty(FGRID_SYMS-1))
 			gBartender->EnableCmd(cmdInsertCell);

 		if (gClipSym != NULL)
 			gBartender->EnableCmd(cmdPaste);
 		
 		gBartender->EnableCmd(cmdSymOperation);				
	}
 }



/***
 * NewFile
 *
 *	When the user chooses New from the File menu, the CreateDocument()
 *	method in your Application class will send a newly created document
 *	this message. This method needs to create a new window, ready to
 *	work on a new document.
 *
 ***/
 
void CFileWindow::NewFile (void)

{    
    Str255  wTitle;     /* Window title string.         */
    short   wCount;     /* Index number of new window.  */
    Str255  wNumber;    /* Index number as a string.    */
    short	symCode;
    
    
		/**
		 **	Initialize all symbols to be empty.
	     **
		 **/
    
    for (symCode = 1; symCode < FGRID_SYMS; symCode++)
    	NewSymbol(symCode); 
   
    
		/**
		 **	Build & display new (empty) file window.
		 **
		 **/

	BuildWindow();
	itsWindow->Select();
	
}



/***
 * OpenFile
 *
 *	When the user chooses Open from the File menu, the OpenDocument()
 *	method in your Application class will let the user choose a file
 *	and then send a newly created document this message. The information
 *	about the file is in the SFReply record.
 *
 ***/

void CFileWindow::OpenFile (SFReply *macSFReply)

{
	CDataFile		*theFile = NULL;
	CProgressWindow *progWind = NULL;
	Str63			theName;
	OSErr			theError;
	short			symCode;
	Handle			symBuf = NULL;
	short			symCount;
	unsigned short 	symBytes;
	
	
	TRY
	{
			/**
			 ** Create a file and send it a SFSpecify()
			 **	message to set up the name, volume, and
			 **	directory.
			 **
			 **/
	
		theFile = new CDataFile;
		
		theFile->IDataFile();
		theFile->SFSpecify(macSFReply);
		
		
			/**
			 **	Be sure to set the instance variable
			 **	so other methods can use the file if they
			 **	need to. This is especially important if
			 **	you leave the file open in this method.
			 **
			 **/
	
		itsFile = theFile;
		
	
			/**
			 **	Send the file an Open() message to open it.
			 **
			 **/
	
		theFile->Open(fsRdWrPerm);
		
	
			/**
			 **	Read number of defined symbols
			 ** and open read-progress window.
			 **
			 **/
			 
		theFile->ReadSome((Ptr) &symCount, sizeof(short));

		progWind = new CProgressWindow;
		progWind->IProgressWindow(this, symCount);
		
		
			/**
			 **	Make sure that the memory request to allocate
			 **	the read buffer doesn't use up any of our rainy
			 ** day fund and that the GrowMemory() method (in 
			 ** the application) knows that it's OK if we 
			 ** couldn't get enough memory.
			 **
			 **/
	
		SetCriticalOperation(FALSE);
		SetAllocation(kAllocCanFail);
		
		symBuf = NewHandle(SYMBUF_SIZE);
		
		
		for (symCode = 1; symCode < FGRID_SYMS; symCode++) {
			
			
			/**
			 **	Read symbol length to determine how many bytes
			 **	to read.  Read symbol bytes into buffer and then
			 **	unpack them into SymEdit's private data structure.
			 **
			 **/
			 
			theFile->ReadSome((Ptr) &symBytes, sizeof(unsigned short));
		
			if (symBytes != 0) {
			
				HLock(symBuf);
				theFile->ReadSome(*symBuf, symBytes);
				HUnlock(symBuf);
			}
		
			UnpackSymbol(symCode, symBuf, symBytes);
			
			progWind->ShowProgress();
												
		}	
		
		 
		progWind->Dispose();
		progWind = NULL;
		
		DisposHandle(symBuf);
		symBuf = NULL;
			 
	
			/**
			 **	Build & display opened file window.
			 **
			 **/
		 
		BuildWindow();
		 
		theFile->GetName(theName);
		itsWindow->SetTitle(theName);
		itsWindow->Select();

	}
	
	CATCH
	{
		 if (symBuf) DisposHandle(symBuf);
		 if (progWind) progWind->Dispose();		 
	}
	ENDTRY;
}



/***
 * DoSave
 *
 *	This method handles what happens when the user chooses Save from the
 *	File menu. This method should return TRUE if the file save was successful.
 *	If there is no file associated with the document, you should send a
 *	DoSaveFileAs() message.
 *
 ***/

Boolean CFileWindow::DoSave(void)

{	
	short			symCode;
	Handle			symBuf = NULL;
	short			symCount;
	unsigned short 	symBytes;
	
	
	if (itsFile == NULL)
		return DoSaveFileAs();
	
		
	TRY
	{
			/**
			 **	Reset file position.
			 **
			 **/
			 
		((CDataFile *) itsFile)->SetMark(0, fsFromStart);
			 
		
			/**
			 **	Alloc write buffer by any means necessary.
			 **
			 **/
			 
		SetCriticalOperation(TRUE);
		SetAllocation(kAllocCantFail);
		
		symBuf = NewHandle(SYMBUF_SIZE);
		 
		
			/**
			 **	Write number of defined symbols.
			 **
			 **/
		
		GetSymbols(&symCount);
		
		((CDataFile *) itsFile)->WriteSome((Ptr) &symCount, sizeof(short));

			
			/**
			 **	For each symbol, pack SymEdit's private data structure into
			 ** buffer and return the number of buffer bytes.
			 ** Write symbol length to file followed by the
			 ** symbol bytes themselves.
			 **
			 **/
			 
		for (symCode = 1; symCode < FGRID_SYMS; symCode++) {

 			PackSymbol(symCode, symBuf, &symBytes);

			((CDataFile *) itsFile)->WriteSome((Ptr) &symBytes, sizeof(unsigned short));
		
			if (symBytes != 0) {
				HLock(symBuf);
				((CDataFile *) itsFile)->WriteSome(*symBuf, symBytes);
				HUnlock(symBuf);
			}
			
		}	
			    
		DisposHandle(symBuf);
		symBuf = NULL;
		
		dirty = FALSE;
		return (TRUE);					/* Save successful */
		
	}
	
	CATCH
	{
		 if (symBuf) DisposHandle(symBuf);		 
	}
	ENDTRY;
	
}



/***
 * DoSaveAs
 *
 *	This method handles what happens when the user chooses Save As from
 *	File menu. The default DoCommand() method for documents sends a DoSaveFileAs()
 *	message which displays a standard put file dialog and sends this message.
 *	The SFReply record contains all the information about the file you're about
 *	to create.
 *
 ***/

Boolean CFileWindow::DoSaveAs(SFReply *macSFReply)

{
		/**
		 **	If there's a file associated with this document
		 **	already, close it. The Dispose() method for files
		 **	sends a Close() message to the file before releasing
		 **	its memory.
		 **
		 **/
		 
	if (itsFile != NULL)
		itsFile->Dispose();


		/**
		 **	Create a new file, and then save it normally.
		 **
		 **/
		 
	SetCriticalOperation(TRUE);
	SetAllocation(kAllocCantFail);
	
	itsFile = new CDataFile;
	
	((CDataFile *) itsFile)->IDataFile();
	itsFile->SFSpecify(macSFReply);
	
	if (itsFile->ExistsOnDisk())
		itsFile->ThrowOut();
		
	itsFile->CreateNew(gSignature, 'SWSY');
	itsFile->Open(fsRdWrPerm);
	
	itsWindow->SetTitle(macSFReply->fName);

	return DoSave();
}
	


/***
 * DoRevert
 *
 *  Close the current file (without writing anything out)
 *	and read the last saved version of the file.
 *
 ***/

void CFileWindow::DoRevert(void)

{
	short			symCode;
	Handle			symBuf = NULL;
	short			notUsed;
	unsigned short 	symBytes;

										/* Only file window can be open		*/
	if (itsGridPane->SymWindowsOpen())
		return;
		
										/* Dispose existing sym data		*/									
	for (symCode = 1; symCode < FGRID_SYMS; symCode++) 
		DisposSymbol(symCode);
		

	itsFile->Close();		
	itsFile->Open(fsRdWrPerm);			/* Reopen file and read sym data	*/
			
									
	TRY
	{									/* Skip symbol count				*/
		((CDataFile *) itsFile)->ReadSome((Ptr) &notUsed, sizeof(short));
	
	
		SetCriticalOperation(FALSE);
		SetAllocation(kAllocCanFail);
		
		symBuf = NewHandle(SYMBUF_SIZE);
		
		
		for (symCode = 1; symCode < FGRID_SYMS; symCode++) {
			
			((CDataFile *) itsFile)->ReadSome((Ptr) &symBytes, sizeof(unsigned short));
		
			HLock(symBuf);
			((CDataFile *) itsFile)->ReadSome(*symBuf, symBytes);
			HUnlock(symBuf);
		
			UnpackSymbol(symCode, symBuf, symBytes);
			
		}		
		 	
		
		DisposHandle(symBuf);
		symBuf = NULL;
		
		
		itsGridPane->Refresh();			/* Update with reverted contents   */	

	}
	
	CATCH
	{
		 if (symBuf) DisposHandle(symBuf);		 
 	}
	ENDTRY;
}



/***
 * Close
 *
 *  Close the current document and dispose 
 *	SymEdit's symbol data structure.
 *
 ***/

Boolean CFileWindow::Close (Boolean quitting)

{
	Boolean 	result = inherited::Close(quitting);
	short		symCode;
	
							/* Dispose existing sym data */			
	if (result)																
		for (symCode = 1; symCode < FGRID_SYMS; symCode++) 
			DisposSymbol(symCode);
			
	return result;
}
		
	

/***
 * DoSymbolOperation
 *
 *	Display a dialog box allowing the user
 *	to test the operation of the currently
 *  selected symbol.
 *
 ***/
 
void CFileWindow::DoSymbolOperation (void)

{  
	short			item = itsGridPane->GetSelection();
	short			symbol, result;
	CFileWndDialog  *dialog;

	// get selected symbol	
	symbol = itsGridPane->ItemToSymCode(item);

	// exit if empty
	if (SymEmpty(symbol)) {
		PositionDialog('ALRT', ALRTnoSymbol);	
		result = Alert(ALRTnoSymbol, NULL);	
		return;
	}
		
	// init dialog class
	dialog = new CFileWndDialog;
	dialog->IFileWndDialog(symbol, this);
		
	// perform dialog
	dialog->SymbolOperation();
	dialog->Dispose();	
}



/***
 * BuildWindow
 *
 *	This is the auxiliary window-building method that the
 *	NewFile() and OpenFile() methods use to create the
 *  symbol file window.
 *
 ***/

void CFileWindow::BuildWindow ()

{
	CScrollPane		*theScrollPane;
	CFileWndGrid	*theMainPane;
	CFileWndBox		*theBoxPane;
	CPaneBorder		*theBoxBorder;
	Rect			aRect;

	   /**
		**	First create the window and initialize
		**	it. The first argument is the resource ID
		**	of the window. The second argument specifies
		**	whether the window is a floating window.
		**	The third argument is the window's enclosure; it
		**	should always be gDesktop. The last argument is
		**	the window's supervisor in the Chain of Command;
		**	it should always be the Document object.
		**
		**/

	itsWindow = new CWindow;
	itsWindow->IWindow(FWND_RSRC, FALSE, gDesktop, this);
	
	
		/**
		 **	After you create the window, you can use the
		 **	SetSizeRect() message to set the windows maximum
		 **	and minimum size. Be sure to set the max & min
		 **	BEFORE you send a PlaceNewWindow() message to the
		 **	decorator.
		 **
		 **/
		 
	SetRect(&aRect, FWND_MINWIDTH, FWND_HEIGHT, FWND_MAXWIDTH, FWND_HEIGHT);
	itsWindow->SetSizeRect(&aRect);


		/**
         ** The symfile window contains a display pane
         ** (for showing the current symbol code) and
         ** also a ScrollPane (which in turn contains
         ** a Panorama.
		 **
		 **/

    theScrollPane = new CScrollPane;
   
	theScrollPane->IScrollPane(itsWindow, this, 0, 0, 0, 0,
								sizELASTIC, sizFIXEDLEFT,
								TRUE, FALSE, TRUE);

		/**
		 **	The FitToEnclFrame() method makes the
		 **	scroll pane be as large as its enclosure.
		 **	In this case, the enclosure is the window,
		 **	so the scroll pane will take up the entire
		 **	window.
		 **
		 **/

	theScrollPane->FitToEnclFrame(TRUE, TRUE);


		/**
		 **	theBoxPane controls the box at the bottom left
		 ** corner of the symfile window.  It is used to 
		 ** display the code value of the symbol currently
		 ** selected in the symfile window.
		 **
		 **/
		 
	theBoxPane = new CFileWndBox;
 	theBoxPane->IFileWndBox(theScrollPane, this,
    						FBOX_WIDTH, FBOX_HEIGHT,
    						0, FWND_HEIGHT - FBOX_HEIGHT, 
    						sizFIXEDLEFT, sizFIXEDBOTTOM);
    						
    itsBoxPane = theBoxPane;

	
		/**
		 **	theBoxBorder is used to draw the top
		 ** border on the box pane.
		 **
		 **/
		 
	theBoxBorder = new CPaneBorder;
	theBoxBorder->IPaneBorder(kBorderTop);
	
	theBoxPane->SetBorder(theBoxBorder);


		/**
		 **	itsMainPane is the document's focus
		 **	of attention. Some of the standard
		 **	classes (particularly CPrinter) rely
		 **	on itsMainPane pointing to the main
		 **	pane of your window.
		 **
		 **	Note that the main pane is the
		 **	panorama in the scroll pane and not
		 **	the scroll pane itself.
		 **
		 **/

	theMainPane = new CFileWndGrid;
    theMainPane->IFileWndGrid(theScrollPane, this,
    						  0, 0, 0, 0, sizELASTIC, sizELASTIC);
    						  
    theMainPane->SetCommandBase(cmdSelectCell);
   						  
	itsGridPane = theMainPane;
	itsMainPane = theMainPane;
	gFileWind   = theMainPane;
	
	
		/**
		 **	The FitToEnclosure() method makes the pane
		 **	fit inside the enclosure. The inside (or
		 **	interior) of a scroll pane is defined as
		 **	the area inside the scroll bars.
		 **
		 **/

	theMainPane->FitToEnclosure(TRUE, TRUE);


		/**
		 **	Send the scroll pane an InstallPanorama()
         ** message to associate the panorama with 
         ** the scroll pane.
		 **
		 **/

	theScrollPane->InstallPanorama(theMainPane);
	
	
		/**
		 **
		 ** The document class fields all file window
		 ** commands.
		 ** 
		 **/
		 
	this->BecomeGopher(TRUE);
}

