
/****
 * CFileWndDialog.c
 *
 *	Operation window methods for the SignWriter image editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/



#include "CFileWndDialog.h"
#include "CApplication.h"
#include "CButton.h"
#include "CPane.h"
#include "CPaneBorder.h"
#include "CWindow.h"
#include "Commands.h"
#include "LongQD.h"
#include "Symbols.h"
#include "SymEditConsts.h"
#include "TBUtilities.h"




 /***************************  Dialog pane methods  **************************/
 
 
  
 
/***
 * Draw
 *
 *  Do initial display of symbol pane.
 *
 ***/

void CSymPane::Draw (Rect *area)

{
	this->Update(itsSymCode, 1, 1, 0, 0);
}



/***
 * Update
 *
 *  Draw symbol image into the dialog
 *  box symbol pane.  Draw border
 *  around the image to show its frame.
 *
 ***/

void CSymPane::Update (short sym, short symVar, short fillVar, short rot, short flop)

{
	LongRect paneRect;
	LongPt   symPos;
	LongPt   imgFrame;
	LongPt   imgCenter;
	Rect	 frame;
	
	
	// erase current image
	
	Prepare();
	
	GetFrame(&paneRect);
	LEraseRect(&paneRect);
	

	// draw sym image
	
	GetCoords(sym, symVar, fillVar, rot, flop, &imgFrame, &imgCenter, FALSE);
	
	symPos.h = (paneRect.right  / 2) - imgCenter.h;
   	symPos.v = (paneRect.bottom / 2) - imgCenter.v;
   	
	DrawSymbol(sym, symVar, fillVar, rot, flop, &thePort->portBits, &symPos, FALSE);		 


	// draw image frame
		
	imgFrame.h += symPos.h;
   	imgFrame.v += symPos.v;
   	
	SetRect(&frame, symPos.h, symPos.v, imgFrame.h, imgFrame.v);
	InsetRect(&frame, -2, -2);
	
	PenPat(gray);
	FrameRect(&frame);

}


  
/***
 * Draw
 *
 *  Do initial display of text pane.
 *
 ***/

void CTxtPane::Draw (Rect *area)

{
	this->Update(0, 0);
} 



/***
 * Update
 *
 *  Display current rotation and flop
 *  values in the dialog box text pane.
 *
 ***/

void CTxtPane::Update (short rot, short flop)

{
	Str15	  msgStr = "Rot X, Flop Y";
	LongRect  rect;
	
	Prepare();
	GetFrame(&rect);
	LEraseRect(&rect);
	
	msgStr[ 4] = '1' + rot  - 1;
	msgStr[12] = '1' + flop - 1;
			
	TextFont(1); TextFace(1); TextSize(10);
	MoveTo(0, 15);
	DrawString(CtoPstr(msgStr));
}



/*****************************  Dialog methods  *******************************/


/***
 * IFileWndDialog
 *
 *  Initialize a new instance of a file window dialog.
 *
 ***/
 
void CFileWndDialog::IFileWndDialog (short symCode, CDirectorOwner *aSupervisor)
	
{	
	Str27			winTitle = "\pSymbol Operation";
	CPaneBorder    *theBorder;
	
	
	// do inherited
	
	CDLOGDirector::IDLOGDirector(DLOGsymOperate, aSupervisor);

			     
	// Translate resource item numbers into button objects
	
	symFill   = (CButton*) itsWindow->FindViewByID (DITLsymFill);
	symMirror = (CButton*) itsWindow->FindViewByID (DITLsymMirror);	
	symChange = (CButton*) itsWindow->FindViewByID (DITLsymChange);
	symRotate = (CButton*) itsWindow->FindViewByID (DITLsymRotate);	 
		
	
	// Translate resource item numbers into dialog panes
	
	symPane = (CSymPane*) itsWindow->FindViewByID (DITLsymPane);
	txtPane = (CTxtPane*) itsWindow->FindViewByID (DITLtxtPane);

	theBorder = new CPaneBorder;
	theBorder->IPaneBorder(kBorderFrame);
	
	symPane->SetBorder(theBorder);
	
	 
	// Assign local command numbers to button objects

	symFill  ->SetClickCmd (DITLsymFill);
	symMirror->SetClickCmd (DITLsymMirror);
	symChange->SetClickCmd (DITLsymChange);
	symRotate->SetClickCmd (DITLsymRotate);
	
	
 	// Initial symbol state
	
	itsSymCode  = symCode;
	itsSymVar   = 1;
	itsFillVar  = 1;
	itsRotation = 0;
	itsFlop	 	= 0;
	
	symPane->itsSymCode  = symCode;
	

	// Initial sym attributes
	
	GetSymVars (symCode, &maxSymVar);
	GetFillVars(symCode, itsSymVar, &maxFillVar);
	
	GetRotations(symCode, itsSymVar, itsFillVar, &maxRotation);
	GetFlops    (symCode, itsSymVar, itsFillVar, &maxFlop);
	GetClass    (symCode, itsSymVar, itsFillVar, &imgClass);


	// Set window title
	
	itsWindow->SetTitle(winTitle);
	
}


 
/***
 * SymbolOperation
 *
 *  Perform modal dialog displaying the
 *  operation of the specified symbol.
 *
 ***/

void CFileWndDialog::SymbolOperation (void)

{
	long  junk;

	BeginDialog();
	junk = DoModalDialog(cmdOK);
}



/***
 * DoCommand (override)
 *
 *  Process buttons during dialog.
 *
 ***/

void CFileWndDialog::DoCommand (long theCommand)

{
	short numRots;
	
	
	switch (theCommand) {
			
			
		case DITLsymChange:
		
			if (!KeyIsDown(OPTKEY_CODE)) {
			
				itsSymVar++;
				if (itsSymVar > maxSymVar)
					itsSymVar = 1;
					
			} else {
			
				itsSymVar--;
				if (itsSymVar < 1)
					itsSymVar = maxSymVar;
			}
				
			break;
			
				
		case DITLsymFill:
		
			if (!KeyIsDown(OPTKEY_CODE)) {
			
				itsFillVar++;
				if (itsFillVar > maxFillVar)
					itsFillVar = 1;
					
			} else {
			
				itsFillVar--;
				if (itsFillVar < 1)
					itsFillVar = maxFillVar;
			}
	
			break;
			
			
		case DITLsymMirror:
			
			numRots = maxRotation + 1;
					
			if (numRots > 3) {
			
				if (imgClass == class3B)
					itsRotation += 4;
					
				itsRotation = (numRots - (itsRotation % numRots)) % numRots;
			}
				
			if (maxFlop == 1)
				itsFlop = (itsFlop + 1) % 2;
			
			break;
				
			
		case DITLsymRotate:
		
			if (!KeyIsDown(OPTKEY_CODE)) {
			
				itsRotation++;
				if (itsRotation > maxRotation)
					itsRotation = 0;
					
			} else {
			
				itsRotation--;
				if (itsRotation < 0)
					itsRotation = maxRotation;
			}
			
			break;
			

		default:
			inherited::DoCommand(theCommand);
			break;
	}
	
	UpdateDisplay();
	
}

 
 
/***
 * UpdateDisplay
 *
 *  Update symbol state and display.
 *  Adjust rot & flop values to match
 *  changing sym & fill var classes.
 *
 ***/

void CFileWndDialog:: UpdateDisplay (void)

{
	short  curRot, curFlop;
	
	
	// update attributes for current sym var
	
	GetFillVars(itsSymCode, itsSymVar, &maxFillVar);			
	if (itsFillVar > maxFillVar)
		itsFillVar = maxFillVar;
	
	
	// update attributes for current sym image class
	
	GetRotations(itsSymCode, itsSymVar, itsFillVar, &maxRotation);
	GetFlops    (itsSymCode, itsSymVar, itsFillVar, &maxFlop);
	GetClass    (itsSymCode, itsSymVar, itsFillVar, &imgClass);
	
	
	// adjust rot & flop values for current sym image class
	
	curRot  = itsRotation % (maxRotation + 1);
	curFlop = itsFlop % (maxFlop + 1);
	
	if (itsFlop == 1 && maxRotation >= 3 && imgClass == class3B)
		curRot = (curRot + 4) % (maxRotation + 1);
	
	
	// redraw symbol & text display
	
	symPane->Update(itsSymCode, itsSymVar, itsFillVar, curRot, curFlop);
	txtPane->Update(curRot, curFlop);

}

