/******************************************************************************
 CImgWndClear.c

							The ImgWndClear Class
		
	SUPERCLASS = CImgWndTask
	
    Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.

 ******************************************************************************/



#include <Global.h>
#include "CImgWndClear.h"



/*** Global Variables ***/

extern long			gSleepTime;			/* Max time between events			*/




/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IImgWndClear

		Initialize an ImgWndClear object
 ******************************************************************************/

void CImgWndClear::IImgWndClear (short aNameIndex, CImgWndPane *aPane, CBitMap *aBitMap)

{
	CImgWndTask::IImgWndTask(aNameIndex, aPane, aBitMap);
	
	done = FALSE;
}


/******************************************************************************
 Do {OVERRIDE}

		Clear the current selection in an EditPane
 ******************************************************************************/

void CImgWndClear::Do()

{
	LongRect	sRect;

	if (!done) {
		done = TRUE;
		
		itsPane->Prepare();
		itsPane->LPaneToBitMapRect(&itsPane->selRect, &sRect);
		SaveForUndo(&sRect);
	
		itsBitMap->BeginDrawing();		/* White out selected area	*/
		LEraseRect(&sRect);
		itsBitMap->EndDrawing();	
										/* Update pane display		*/
		itsPane->Update(itsBitMap->macBitMap, &sRect);
		
		itsPane->KillSelRect();			/* Zap selection			*/
	}
}


/******************************************************************************
 Undo {OVERRIDE}

		Undo clearing of a selection
 ******************************************************************************/

void	CImgWndClear::Undo()

{
	LongRect	bounds;
	
	inherited::Undo();
										/* If there's no current selection,	*/
										/*   select the area which had been	*/
										/*   cleared						*/
										
	if (EmptyLongRect(&itsPane->selRect)) {
		itsUndoBits->GetBounds(&bounds);
		itsPane->LBitMapToPaneRect(&bounds, &itsPane->selRect);
		
		gSleepTime = 0;					/* Need to force idle to march ants	*/
	}
}


/******************************************************************************
 Redo {OVERRIDE}

		Redo clearing of a selection
 ******************************************************************************/

void	CImgWndClear::Redo()

{
	LongRect	bounds;
	LongRect	sRect;
	
	inherited::Undo();

	itsUndoBits->GetBounds(&bounds);	/* If cleared area is selected,		*/
										/*   kill it so we don't march ants	*/
										/*   around the nuked space			*/
	itsPane->LPaneToBitMapRect(&itsPane->selRect, &sRect);
							
	if (EqualLongRect(&bounds, &sRect)) {
		itsPane->Update(itsBitMap->macBitMap, &sRect);
		SetLongRect(&itsPane->selRect, 0, 0, 0, 0);	   // Set empty selection
	}
}