/******************************************************************************
 CImgWndSelDrag.c

							The CImgWndSelDrag Class
		
	SUPERCLASS = CImgWndTask
	
    Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.

 ******************************************************************************/


#include <Global.h>
#include <TBUtilities.h>
#include <CDocument.h>
#include "CImgWndSelDrag.h"
#include "SymEditConsts.h"



extern CBitMap		*gScratchPad;





/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 ISelDrag

		Initialize a Drag object
 ******************************************************************************/

void	CSelDrag::ISelDrag(
	CImgWndPane	*aPane,
	CBitMap		*aBitMap,
	short		 aCmd)
{
	CImgWndTask::IImgWndTask(aCmd, aPane, aBitMap);
	
	dragCmd = aCmd;
	newDrag = TRUE;
	done    = FALSE;
	undone  = FALSE;
}


/******************************************************************************
 BeginTracking
 ******************************************************************************/

void	CSelDrag::BeginTracking(
	LongPt		*startPt)
{
	LongRect	currRect;
	LongRect	undoSel;

	if (newDrag) {						/* A first time drag				*/
		newDrag = FALSE;
		itsPane->LPaneToBitMapRect(&itsPane->selRect, &undoSel);
		
		currRect = undoSel;
		this->undoSel = undoSel;
		
		SaveForUndo(&undoSel);			/* Save selection in undo bit map	*/
		
					 					/* Option key not down, we clear	*/
					 					/*   the area under the selection	*/
		if (!KeyIsDown(OPTKEY_CODE)) {
		
			itsBitMap->BeginDrawing();
			LEraseRect(&undoSel);
			itsBitMap->EndDrawing();
			cloned = FALSE;
			
		} else {						/* Option key down, so we save		*/
			cloned = TRUE;				/*    the existing selection		*/
		}

	} else {							/* Dragging again after a delay		*/
										/*   Selection is already "wet"		*/
										
		itsPane->LPaneToBitMapRect(&itsPane->selRect, &currRect);
		
		LCopyBits(gScratchPad->macBitMap, itsBitMap->macBitMap,
					&currRect, &currRect, srcCopy, NULL);
					
		if (undone) {					/* Correct wording of Undo menu		*/
										/*   item							*/
			((CDocument*) itsPane->itsSupervisor)->undone = FALSE;
			undone = FALSE;
		}
	}
}


/******************************************************************************
 KeepTracking
 ******************************************************************************/

void	CSelDrag::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt		currMapPt;
	LongPt		prevMapPt;
	LongPt		startMapPt;
	
	LongRect	currRect;
	LongRect	prevRect;
	LongRect	changedRect;
	LongRect	bounds;

										// Convert to bitmap coordinates
	itsPane->PaneToBitMapPt(currPt, &currMapPt);
	itsPane->PaneToBitMapPt(prevPt, &prevMapPt);
	itsPane->PaneToBitMapPt(startPt, &startMapPt);
	
	if (!EqualLongPt(&currMapPt, &prevMapPt)) {	
										
										/* Don't allow selection to be	*/
										/*   dragged outside the pane  	*/
		itsPane->LPaneToBitMapRect(&itsPane->selRect, &prevRect);
		
		itsBitMap->GetBounds(&currRect);
		currRect.left += startMapPt.h - prevRect.left;
		currRect.top += startMapPt.v - prevRect.top;
		currRect.right += startMapPt.h - prevRect.right;
		currRect.bottom += startMapPt.v - prevRect.bottom;
		
		PinInRect(&currRect, &currMapPt);
		itsPane->BitMapToPanePt(&currMapPt, currPt);  // update param value

		currRect = prevRect;	
		OffsetLongRect(&currRect, currMapPt.h - startMapPt.h, currMapPt.v - startMapPt.v);
		OffsetLongRect(&prevRect, prevMapPt.h - startMapPt.h, prevMapPt.v - startMapPt.v);
		UnionLongRect(&currRect, &prevRect, &changedRect);
					  
										/* Copy changed area to scratch pad	*/
		LCopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
				 &changedRect, &changedRect, srcCopy, NULL);

										/* Copy selection to scratch pad	*/
		QDToLongRect(&itsUndoBits->macBitMap->bounds, &bounds);
		LCopyBits(itsUndoBits->macBitMap, gScratchPad->macBitMap,
				 &bounds, &currRect, srcCopy, NULL);
				 
										/* Update pane display				*/
		itsPane->Update(gScratchPad->macBitMap, &changedRect);
	}
}


/******************************************************************************
 EndTracking
 ******************************************************************************/

void	CSelDrag::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongRect	currRect;
	LongRect	undoSel;
	LongPt		currMapPt;
	LongPt		startMapPt;

	
	KeepTracking(currPt, prevPt, startPt);
	
	itsPane->LPaneToBitMapRect(&itsPane->selRect, &currRect);
	
	itsPane->PaneToBitMapPt(currPt, &currMapPt);
	itsPane->PaneToBitMapPt(startPt, &startMapPt);
	
	OffsetLongRect(&currRect, currMapPt.h - startMapPt.h,
						  	  currMapPt.v - startMapPt.v);
						  
	itsPane->LBitMapToPaneRect(&currRect, &itsPane->selRect);

	itsPane->Prepare();
										/* Save area under where selection	*/
										/*   was dragged so we can undo		*/
	LCopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
				&currRect, &currRect, srcCopy, NULL);
			
	undoSel = this->undoSel;			/* The selected area is actually in	*/
										/*   the undo bits now. Update		*/
										/*   bitmap to reflect new			*/
										/*   location of selection.			*/
	LCopyBits(itsUndoBits->macBitMap, itsBitMap->macBitMap,
				&undoSel, &currRect, srcCopy, NULL);
				
										/* At this point, the paint is		*/
										/*   still "wet". The selection is	*/
										/*   in the undo bits and the		*/
										/*   scratchpad holds what was		*/
										/*   underneath the selection.		*/
}


/******************************************************************************
 Do {OVERRIDE}
 
 	"Dry" the paint of the dragged area
 ******************************************************************************/

void	CSelDrag::Do()
{
	LongRect	currRect;

	if (!done) {
		itsPane->LPaneToBitMapRect(&itsPane->selRect, &currRect);

		itsPane->Prepare();
										/* Save bits under selection		*/
		itsUndoBits->SetBoundsOrigin(currRect.left, currRect.top);
		LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
				 &currRect, &currRect, srcCopy, NULL);
		
		done = TRUE;
	}
}


/******************************************************************************
 Undo {OVERRIDE}

		Undo drag
 ******************************************************************************/

void	CSelDrag::Undo()
{
	LongRect	currRect;
	LongRect	undoSel;
	
	if (itsUndoBits != NULL) {			/* Make sure we can undo			*/
		undone = TRUE;
		
		if (done) {						/* Paint has dried					*/
			
										/* Get current and former locations	*/
			itsUndoBits->GetBounds(&currRect);
			undoSel = this->undoSel;
			
			itsPane->Prepare();
										/* Copy both the old and new select	*/
										/*   areas to the scratchpad		*/
			LCopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
						&currRect, &currRect, srcCopy, NULL);
			LCopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
						&undoSel, &undoSel, srcCopy, NULL);
						
										/* Put undo area back in image		*/
			LCopyBits(itsUndoBits->macBitMap, itsBitMap->macBitMap,
						&currRect, &currRect, srcCopy, NULL);

			if (dragCmd == iPASTE) {
										/* Put new select into undo bits    */
				itsUndoBits->SetBoundsOrigin(currRect.left, currRect.top);
				LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
						 &currRect, &currRect, srcCopy, NULL);

			} else {			
										/* Old selected area now goes into	*/
										/*    undo bits						*/
				itsUndoBits->SetBoundsOrigin(undoSel.left, undoSel.top);
				LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
						 &undoSel, &undoSel, srcCopy, NULL);
						 
					 					/* Put selection back in its old	*/
					 					/*   location						*/
				LCopyBits(gScratchPad->macBitMap, itsBitMap->macBitMap,
						 &currRect, &undoSel, srcCopy, NULL);
				
			}			 
					 					/* Update pane display				*/
			itsPane->Update(itsBitMap->macBitMap, &currRect);
			itsPane->Update(itsBitMap->macBitMap, &undoSel);

			this->undoSel = currRect;
		
		} else {						/* Paint is still wet				*/
										/* Undo bits hold the selection		*/
			undoSel = this->undoSel;
			itsPane->LPaneToBitMapRect(&itsPane->selRect, &currRect);
			
										/* Update pane display				*/
			itsPane->Prepare();
			
			LCopyBits(gScratchPad->macBitMap, itsBitMap->macBitMap,
						&currRect, &currRect, srcCopy, NULL);
						
			itsPane->Update(itsBitMap->macBitMap, &currRect);

										/* zap selection after Paste undo   */
			if (dragCmd == iPASTE) {
				SetLongRect(&itsPane->selRect, 0, 0, 0, 0);
				
			} else {					/* restore it after Drag undo		*/
				itsPane->Update(itsUndoBits->macBitMap, &undoSel);
				
				LCopyBits(itsUndoBits->macBitMap, itsBitMap->macBitMap,
							&undoSel, &undoSel, srcCopy, NULL);
							
				itsPane->LBitMapToPaneRect(&undoSel, &itsPane->selRect);
			}
		
			redoSel = currRect;
		}
	}
}


/******************************************************************************
 Redo {OVERRIDE}

		Redo drag
 ******************************************************************************/

void	CSelDrag::Redo()
{
	LongRect		undoSel;
	LongRect		redoSel;
	
	undone = FALSE;
	if (done) {							/* Same as Undo for dry paint		*/
		Undo();
		
	} else {							/* Dragged area is still wet		*/
		itsPane->Prepare();

		undoSel = this->undoSel;
		redoSel = this->redoSel;
		
		if (!cloned) {					/* No clone, so old area is white	*/
			itsBitMap->BeginDrawing();
			LEraseRect(&undoSel);
			itsBitMap->EndDrawing();
		}
		
		itsPane->Prepare();				/* Undo bits and the painting hold	*/
										/*   all the original info, it's	*/
										/*   just been moved around.		*/
		itsPane->Update(itsBitMap->macBitMap, &undoSel);
		
		LCopyBits(itsUndoBits->macBitMap, itsBitMap->macBitMap,
				 &undoSel, &redoSel, srcCopy, NULL);
		
		itsPane->Update(itsBitMap->macBitMap, &redoSel);
		itsPane->LBitMapToPaneRect(&redoSel, &itsPane->selRect);
	}	
}
