/******************************************************************************
 CPaintTask.c

							The PaintTask Class
		
	SUPERCLASS = CMouseTask
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <TCLUtilities.h>
#include "CPaintTask.h"

/*** Global Variables ***/

extern CBitMap		*gScratchPad;


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IPaintTask

		Initialize a PaintTask object
 ******************************************************************************/

void	CPaintTask::IPaintTask(
	short		aNameIndex,
	CPaintPane	*aPaintPane,
	CBitMap		*aPainting)
{
	inherited::IMouseTask(aNameIndex);

	itsPaintPane = aPaintPane;
	itsPainting = aPainting;
	itsUndoBits = NULL;
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a PaintTask object
 ******************************************************************************/

void	CPaintTask::Dispose()
{
	if (itsUndoBits != NULL) {
		itsUndoBits->Dispose();
	}
	if (itsPaintPane && (itsPaintPane->itsLastTask == this))
	{
		itsPaintPane->itsLastTask = NULL;
	}
	
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 Undo {OVERRIDE}

		Undo an action performed by a paint task
 ******************************************************************************/

void	CPaintTask::Undo()
{
	LongRect	bounds;
	
	if (itsUndoBits != NULL) {
	
		itsPaintPane->Prepare();
		itsUndoBits->GetBounds(&bounds);
		
		LCopyBits(itsPainting->macBitMap, gScratchPad->macBitMap,
					&bounds, &bounds, srcCopy, NULL);
					
		LCopyBits(itsUndoBits->macBitMap, itsPainting->macBitMap,
					&bounds, &bounds, srcCopy, NULL);
					
		LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
					&bounds, &bounds, srcCopy, NULL);
					
		itsPaintPane->Prepare();
		itsPainting->CopyFrom(&bounds, &bounds, NULL);
	}
}


/******************************************************************************
 SaveForUndo

		Save an area of the Painting so a change can be undone
 ******************************************************************************/

void	CPaintTask::SaveForUndo(
	LongRect		*undoRect)
{
	Boolean		savedAlloc;
									/* Put changed area of painting into	*/
									/*   an undo bitmap						*/
	itsUndoBits = new(CBitMap);
	
	savedAlloc = SetAllocation( kAllocCanFail);
	itsUndoBits->IBitMap(undoRect->right - undoRect->left,
						 undoRect->bottom - undoRect->top, FALSE);
	SetAllocation( savedAlloc);
	
	FailNIL( itsUndoBits);

	itsUndoBits->SetBoundsOrigin(undoRect->left, undoRect->top);
	
	itsPaintPane->Prepare();
		
	LCopyBits(itsPainting->macBitMap, itsUndoBits->macBitMap,
					undoRect, undoRect, srcCopy, NULL);
}