/******************************************************************************
 CStringTableDemo.c

		
	SUPERCLASS = CDLOGDirector
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CStringTableDemo.h"
#include "CStringTable.h"
#include "CStringArray.h"
#include "CScrollPane.h"
#include "CDialogText.h"
#include "CDialog.h"
#include "CPaneBorder.h"
#include "DemoCommands.h"
#include "CApplication.h"

#define	kDemoDlgID			1030

enum		/* window item numbers	*/
{
	kAddItem = 1,
	kChangeItem,
	kDeleteItem,
	kEditItem,
	kListScrollPaneItem
};

extern CApplication	*gApplication;

/******************************************************************************
 IStringTableDemo
******************************************************************************/

void CStringTableDemo::IStringTableDemo( void)
{
	CScrollPane	*scrollPane;
	CPaneBorder *listBorder;
	Rect	margin;
	Cell	aCell;
	
	CDLOGDirector::IDLOGDirector( kDemoDlgID, gApplication);
	
	scrollPane = (CScrollPane*) itsWindow->FindViewByID( kListScrollPaneItem);
	
	((CDialog*)itsWindow)->SetDefaultCmd( cmdAddString);
	
	itsEditItem = (CDialogText*) itsWindow->FindViewByID( kEditItem);
	
	if (scrollPane)
	{				
		itsStringTable = new( CStringTable);
		itsStringTable->IStringTable( scrollPane, itsWindow, 0, 0, 0, 0,
			sizELASTIC, sizELASTIC);
		itsStringTable->FitToEnclosure( TRUE, TRUE);
		itsStringTable->SetDrawActiveBorder( TRUE);
		
		itsStringTable->SetID( 10);

		listBorder = new( CPaneBorder);
		listBorder->IPaneBorder( kBorderFrame);
		itsStringTable->SetBorder( listBorder);
				
		itsStringTable->SetSelectionFlags( selOnlyOne);
		
		scrollPane->InstallPanorama( itsStringTable);
		
		itsStrings = new( CStringArray);
		itsStrings->IRes( 1024, 255);
		itsStringTable->SetArray( itsStrings, FALSE);
		
		SetCell( aCell, 0, 0);
		itsStringTable->SelectCell( aCell, FALSE, FALSE);

	}
	
	SetupItems();

}	/* CStringTableDemo::IStringTableDemo */

/******************************************************************************
 DoCommand
******************************************************************************/

void CStringTableDemo::DoCommand( long aCmd)
{
	Cell	selectedCell;
	Str255	string;
	Boolean haveSelection;
	
	SetCell( selectedCell, 0, 0);
	haveSelection = itsStringTable->GetSelect( TRUE, &selectedCell);
	
	switch (aCmd)
	{
		case cmdAddString:
				itsEditItem->GetTextString( string);
				
				/* if a cell is selected, add new string following	*/
				/* the selection, otherwise add to the end. Then	*/
				/* selected the new cell							*/
				
				if (haveSelection)
					selectedCell.v++;
				else
					selectedCell.v = itsStrings->GetNumItems();
					
				itsStrings->InsertAtIndex( string, selectedCell.v+1);
				itsStringTable->SelectCell( selectedCell, FALSE, TRUE);
				itsStringTable->ScrollToSelection();
				break;
				
		case cmdChangeString:
				itsEditItem->GetTextString( string);
				itsStrings->SetItem( string, selectedCell.v+1);
				break;
				
		case cmdDeleteString:
				itsStrings->DeleteItem( selectedCell.v + 1);
				break;
				
		case cmdSortAscending:
				itsStrings->Sort( TRUE);
				SetupItems();
				break;
				
		case cmdSortDescending:
				itsStrings->Sort( FALSE);
				SetupItems();
				break;
				
		default: 
				inherited::DoCommand( aCmd);
				break;
	}
				
}	/* CStringTableDemo::DoCommand */

/******************************************************************************
 SetupItems
******************************************************************************/

void CStringTableDemo::SetupItems( void)
{
	Cell	selectedCell;
	Boolean haveSelection;
	CDialog	*dialog;
	Str255	str;

	/* determine if any cells are now selected	*/
	
	SetCell( selectedCell, 0, 0);
	haveSelection = itsStringTable->GetSelect( TRUE, &selectedCell);
	
	dialog = (CDialog*) itsWindow;
	dialog->SetCmdEnable( cmdDeleteString, haveSelection);
	dialog->SetCmdEnable( cmdChangeString, haveSelection);
	
	if (haveSelection)
		itsStrings->GetItem( str, selectedCell.v+1);
	else
		str[0] = 0;
		
	itsEditItem->SetTextString( str);
	itsEditItem->SelectAll( kRedraw);

}	/* CStringTableDemo::SetupItems */

/******************************************************************************
 ProviderChanged
******************************************************************************/

void CStringTableDemo::ProviderChanged( CCollaborator *aProvider, long reason,
										void *info)
{
	Cell	selectedCell;
	Boolean haveSelection;

	if ((aProvider == itsStringTable) && (reason == tableSelectionChanged))
	{
		SetupItems();
	}
	else
		inherited::ProviderChanged( aProvider, reason, info);

}	/* CStringTableDemo::ProviderChanged */

/******************************************************************************
 Dispose
******************************************************************************/

void CStringTableDemo::Dispose( void)
{
	ForgetObject( itsStrings);
	inherited::Dispose();
}

