
/*
 *  file.c
 *
 *  Copyright (c) 1991 Symantec Corporation.  All rights reserved.
 *
 */

#include <MacHeaders>

#include "stdio.h"
#include "string.h"
#include "errno.h"
#include "ansi_private.h"


static char *gen(char *, char *, int *);


/*
 *  remove - delete file
 *
 */

int
remove(const char *s)
{
	IOParam pb;
	char buf[FILENAME_MAX];
	
	pb.ioNamePtr = __c2p(s, buf);
	pb.ioVRefNum = 0;
	pb.ioVersNum = 0;
	if (PBDeleteSync(&pb) != noErr) {
		errno = pb.ioResult;
		return(-1);
	}
	return(0);
}


/*
 *  rename - rename file
 *
 */

int
rename(const char *s, const char *t)
{
	IOParam pb;
	char oldfile[FILENAME_MAX], newfile[FILENAME_MAX];
	
	pb.ioNamePtr = __c2p(s, oldfile);
	pb.ioVRefNum = 0;
	pb.ioVersNum = 0;
	pb.ioMisc = (Ptr) __c2p(t, newfile);
	if (PBRenameSync(&pb) != noErr) {
		errno = pb.ioResult;
		return(-1);
	}
	return(0);
}


/*
 *  tmpfile - create a scratch file
 *
 *  The file will automatically be deleted when it is closed.
 *
 */

FILE *
tmpfile(void)
{
	static int index;
	char buf[L_tmpnam];
	register FILE *fp;
	
	if (gen(buf, "scratch", &index) && (fp = fopen(buf, "wb+"))) {
		fp->remove = 1;
		return(fp);
	}
	return(NULL);
}


/*
 *  tmpnam - generate unique name
 *
 */

char *
tmpnam(char *s)
{
	static char buf[L_tmpnam];
	static int index;
	
	if (s == NULL)
		s = buf;
	return(gen(s, "temp", &index));
}


/*
 *  gen - generate unique name (common code)
 *
 */

static char *
gen(char *buf, char *root, int *index)
{
	int start = *index;
	FileParam pb;
	
	buf[0] = strlen(root) + 4;
	pb.ioNamePtr = (StringPtr) buf;
	pb.ioVRefNum = 0;
	pb.ioFVersNum = 0;
	pb.ioFDirIndex = 0;
	
		/*  cycle looking for available file name  */
		
	do {
		if (++*index > 9999)
			*index = 1;
		if (*index == start)
			return(NULL);
		sprintf(&buf[1], "%s%.4d", root, *index);
	} while (PBGetFInfoSync(&pb) != fnfErr);
	
		/*  got it - convert to C string  */

	return(memmove(&buf[0], &buf[1], Length(buf) + 1));
}
