/******************************************************************************
 CApplication.h

		Interface for the Application Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- added MakeSwitchBoard, MakeBartender, and MakeError methods
		- removed eventLoopJump instance variable
		- added Process1Event, PackageAppleEvent, DoAppleEvent method
		- added phase instance variable, phase enumeration constants,
		  and GetPhase methods. The application is always in one of three
		  phases: initializing, running, or quitting.
		- changed declaration of sfFileFilter, sfGetDLOGFilter and sfGetDLOGHook 
		  for new header compatibility.
		- added class variable cMaxSleepTime, which determines the default
		  maximum sleep time passed to WaitNextEvent
		- changed interface for IApplication, InitMemory, and RequestMemory methods
		- added SetCriticalOperation method
		- removed instance variables loanApproved and creditLimit
		- added instance variables criticalBalance, toolboxBalance,
		  and tempAllocation.
		- added methods InstallPatches, RemovePatches, ForceClassReferences
	]

 ******************************************************************************/
 
#define _H_CApplication

#include "CDirectorOwner.h"				/* Interface for its superclass		*/

	// Forward declarations;
	
class CSwitchboard;
class CAppleEvent;
class CChore;
class CList;
class CCluster;
struct AppleEvent;

class CApplication : public CDirectorOwner 
{
public:
								/** Instance Variables **/
	CSwitchboard	*itsSwitchboard;	/* Retrieves and processes events	*/
	CList			*itsIdleChores;		/* Chores to perform at idle time	*/
	CCluster		*itsUrgentChores;	/* Chores to perform ASAP			*/
	Boolean			urgentsToDo;		/* Are any urgent chores pending?	*/
	Boolean			running;			/* Status flag						*/
	short			phase;				/* what phase is the application in? */
									/** Rainy day memory reserve **/
	Size			rainyDayFund;		/* Bytes of memory to set aside		*/
	Size			criticalBalance;	/* bytes to save for critical operations */
	Size			toolboxBalance;		/* bytes to save for the toolbox	*/
	Size			tempAllocation;		/* bytes of temporarily allocate mem */
	Handle			rainyDay;			/* Handle to the reserve memory		*/
	Boolean			rainyDayUsed;		/* Has rainy day fund been tapped?	*/
	Boolean			memWarningIssued;	/* Has user been alerted?			*/
	Boolean			canFail;			/* OK for memory request to fail?	*/
	Boolean			inCriticalOperation; /* OK to use critical memory reserve? */
									/** Standard Get File Paramaters **/
	short			sfNumTypes;			/* Number of file types recognized	*/
	SFTypeList		sfFileTypes;		/* File types which are recognized	*/
	FileFilterProcPtr	sfFileFilter;	/* Filter for files to display		*/
	DlgHookProcPtr		sfGetDLOGHook;	/* Hook for handling get dialog		*/
	short			sfGetDLOGid;		/* Dialog resource ID for get file	*/
	ModalFilterProcPtr	sfGetDLOGFilter;/* Filter for get dialog events		*/
	CTask			*unhandledTask;		/* Task that no document handled	*/
	
	static long 	cMaxSleepTime;		/* default for WaitNextEvent		*/
	
	
								/** Instance Methods **/
									/** Initialization **/
	void	IApplication(short extraMasters, Size aRainyDayFund,
							Size aCriticalBalance, Size aToolboxBalance);
	virtual void	InitToolbox(void);
	virtual void	InitMemory(short extraMasters, Size aRainyDayFund,
						Size aCriticalBalance, Size aToolboxBalance);
	virtual void	InstallPatches( void);
	virtual void	RemovePatches( void);
	virtual void	InspectSystem( void);
	virtual void	MakeDesktop(void);
	virtual void	MakeClipboard(void);
	virtual void	MakeDecorator(void);
	virtual void	MakeSwitchboard( void);
	virtual void	MakeBartender( void);
	virtual void	MakeError( void);
	virtual void	SetUpFileParameters(void);
	virtual void	SetUpMenus(void);
	
	virtual void	ForceClassReferences( void);

									/** Action **/
	virtual void	Notify(CTask *theTask);
	virtual void	DoKeyDown(char theChar, Byte keyCode, EventRecord *macEvent);
	virtual void	DoAutoKey(char theChar, Byte keyCode, EventRecord *macEvent);
	virtual void	DoKeyUp(char theChar, Byte keyCode, EventRecord *macEvent);
	virtual void	DoCommand(long theCommand);
	virtual void	UpdateMenus(void);
	
	virtual CAppleEvent *PackageAppleEvent( struct AppleEvent *theEvent, 
						struct AppleEvent *theReply, long theRefCon,
						long eventClass, long eventID);
 
 						// eventClass and eventID are really DescType,
 						// but we don't want to force everyone to include
 						// AppleEvents.h et. al.
						
	virtual void	DoAppleEvent( CAppleEvent *anAppleEvent);

	
									/** Memory Management **/
	virtual void	RequestMemory( Boolean aCanFail);
	virtual void	SetCriticalOperation( Boolean inCriticalOperation);
	virtual long	GrowMemory(Size bytesNeeded);
	virtual void	MemoryShortage(Size bytesNeeded);
	virtual void	MemoryReplenished(void);
	virtual long	OutOfMemory(Size bytesNeeded); 

									/** Execution **/
	virtual void	Run(void);
	virtual void	Process1Event( void);
	virtual void	Preload(void);
	virtual void	StartUpAction(short numPreloads);
	virtual void	Suspend(void);
	virtual void	Resume(void);
	virtual void	SwitchToDA(void);
	virtual void	SwitchFromDA(void);
	virtual void	Idle(EventRecord *macEvent);
	virtual Boolean	Quit(void);
	virtual void	Exit(void);
	virtual void	JumpToEventLoop(void);
	virtual short	GetPhase( void);
	
									/** Document Handling **/
	virtual void	CreateDocument(void);
	virtual void	OpenDocument(SFReply *macSFReply);
	virtual void	DoOpenOrPrintDocEvent( CAppleEvent *theEvent);
	virtual void	ChooseFile(SFReply *macSFReply);
		
									/** Chore Handling **/
	virtual void	AssignIdleChore(CChore *theChore);
	virtual void	CancelIdleChore(CChore *theChore);
	virtual void	AssignUrgentChore(CChore *theChore);
};

enum	/* phases of application execution, return by GetPhase() method */
{
	appInitializing,
	appRunning,
	appQuitting
	
};
