/******************************************************************************
 CArray.h

		Interface for Array class
		
	SUPERCLASS = CCollection
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#define _H_CArray

#include "CCollection.h"
#include "Global.h"

typedef	int (*CompareFunc)( void*, void*);	 /* array element comparison function */

class CArray : public CCollection
{
public:

	short		blockSize;				/* Number of slots to allocate when */
										/*  more space is needed			*/
	long		slots;					/* Total number of slots allocated	*/
	Handle		hItems;					/* Items in the array				*/
	
	long 		elementSize;			/* size of each element in bytes	*/
	Boolean		lockChanges;			/* can't insert or delete if locked */
	Boolean		usingTemporary;			/* TRUE if temporary element storage*/
										/* buffer is in use					*/
		
	void	IArray( long elementSize);
	virtual void	Dispose( void);
	virtual void	SetBlockSize( short aBlockSize);	
	virtual void	InsertAtIndex( void *itemPtr, long index);
	virtual void	DeleteItem( long index);
	virtual void	MoveItemToIndex( long currentIndex, long newIndex);
	virtual void	SetItem( void *itemPtr, long index);
	virtual void	GetItem( void *itemPtr, long index);
	virtual void	Swap( long index1, long index2);
	virtual long	Search( void *itemPtr, CompareFunc compare);
	virtual Boolean SetLockChanges( Boolean fLockChanges);
	virtual CObject	*Copy(void);


	virtual void	Resize( long numSlots);
	virtual void	MoreSlots( void);
	virtual void	Store( void *itemPtr, long index);
	virtual void	Retrieve( void *itemPtr, long index);
	virtual void	CopyToTemporary( long index);
	virtual void	CopyFromTemporary( long index);
	virtual long	ItemOffset( long itemIndex);

};

	/*
	 * A pointer to a tMovedElementInfo structure is passed as the info
	 * parameter to BroadCastChange when MoveItemToIndex has completed
	 */
	 
typedef struct tMovedElementInfo
{
	long	originalIndex;	/* item's original index	*/
	long	newIndex;		/* item's new index			*/
	
} tMovedElementInfo;

/* Change protocol for Array class */

enum									/* info parameter usage		*/
{
	arrayInsertElement = 1,				/* index of new element		*/
	arrayDeleteElement,					/* index of deleted element	*/
	arrayMoveElement,					/* pointer to tMovedElementInfo, see above */
	arrayElementChanged,				/* index of changed element	*/
	
	arrayLastChange = arrayElementChanged
};

#define		BAD_INDEX		-1L			/* Flag indicating a failed search	*/
