/******************************************************************************
 CPane.h

		Interface for the Pane Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- added GetWindow method
		- added instance variable itsBorder, and SetBorder, GetBorder,
		  SetResBorder, and RefreshBorder methods
		- Moved TrackMouse from CView to CPane.
		- for 32-bit support, changed instance variables hEncl, vEncl, 
		  frame, and aperture to use long coordinates. Also changed 
		  interface for SetFrameOrigin, GetFrame, GetAperture, Place, 
		  Offset, EnclosureScrolled, WindToFrame, WindToFrameR, FrameToWind, 
		  FrameToWindR, EnclToFrame, EnclToFrameR, FrameToEncl, FrameToEnclR, 
		  and FrameToGlobalR methods to use long coordinates.
		- added new methods RefreshLongRect, QDToFrame, QDToFrameR, 
		  FrameToQD, FrameToQDR, and SectAperture, GetHelpResID
	]
	
 ******************************************************************************/
 
#define _H_CPane

#include "CView.h"						/* Interface for its superclass		*/


		/** How pane changes size when the size of its enclosure changes **/
		
typedef enum SizingOption	{
			sizFIXEDLEFT,				/* Fixed length, anchored to left	*/
			sizFIXEDRIGHT,				/* Fixed length, anchored to right	*/
			sizFIXEDTOP,				/* Fixed length, anchored to top    */
			sizFIXEDBOTTOM,				/* Fixed length, anchored to bottom	*/
			sizFIXEDSTICKY,				/* Fixed length, sticks to coords	*/
										/*   of its enclosure				*/
			sizELASTIC					/* Variable length, always a fixed	*/
										/*   amount smaller than enclosure	*/
} SizingOption;

typedef enum ClipOption	{
			clipAPERTURE,
			clipFRAME,
			clipPAGE
} ClipOption;

	// Forward declarations
	
class CPrinter;
class CPaneBorder;
class CEnvironment;
class CMouseTask;
class CPaneBorder;
class CWindow;

class CPane : public CView {			/* Class Declaration				*/

public:
								/** Instance Variables **/
	short			width;				/* Horizontal size in pixels		*/
	short			height;				/* Vertical size in pixels			*/
	long			hEncl;				/* Horizontal location in enclosure	*/
	long			vEncl;				/* Vertical location in enclosure	*/
	SizingOption	hSizing;			/* Horizontal sizing option			*/
	SizingOption	vSizing;			/* Vertical sizing option			*/
	Boolean			autoRefresh;		/* Refresh all after a resize?		*/
	LongRect		frame;				/* Area for displaying the Pane		*/
										/*   which defines Frame coords		*/
	LongRect		aperture;			/* Active drawing area of the Pane	*/
	long			hOrigin;			/* Window left in Frame coords		*/
	long			vOrigin;			/* Window top in Frame coords		*/
	CEnvironment	*itsEnvironment;	/* Drawing environment				*/
	ClipOption		printClip;			/* Clipping option when printing	*/
	Boolean			printing;			/* Is printing in progress?			*/
	CPaneBorder 	*itsBorder;			/* border of this pane				*/
	
	CTask		  	*itsLastTask;		/* subclasses should always make	*/
										/* sure that itsLastTask points to	*/
										/* the last CTask subclass created	*/
										/* by the pane. The Dispose() 		*/
										/* method will make sure that the	*/
										/* task is disposed when the pane	*/
										/* is disposed. This is important	*/
										/* because most tasks refer back to	*/
										/* the originating pane and would	*/
										/* be left with a dangling pointer.	*/
	
	static	Rect	cPageArea;			/* area of page being printed		*/
	
					
								/** Instance Methods **/
								
									/** Construct/Destruction **/
	void	IPane(CView *anEnclosure, CBureaucrat *aSupervisor,
					short aWidth, short aHeight, short aHEncl, short aVEncl,
					SizingOption aHSizing, SizingOption aVSizing);
	virtual void	IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
						Ptr viewData);
	void	IPaneX(void);
	virtual void	Dispose(void);
	
									/** Accessing **/
	virtual void	SetFrameOrigin(long fLeft, long fTop);
	virtual void	GetFrame(LongRect *theFrame);
	virtual void	GetLengths(short *theWidth, short *theHeight);
	virtual void	GetOrigin(long *theHOrigin, long *theVOrigin);
	virtual void	GetAperture(LongRect *theAperture);
	virtual Boolean	Contains(Point windPt);
	virtual Boolean ReallyVisible(void);
	virtual void	GetPixelExtent(long *hExtent, long *vExtent);
	virtual void	SetPrintClip(ClipOption aPrintClip);
	virtual CWindow	*GetWindow( void);
	virtual void	SetBorder( CPaneBorder *aBorder);
	virtual void	SetResBorder( short resID);
	virtual CPaneBorder *GetBorder( void);
	virtual short  GetHelpResID( void);

									/** Display **/
	virtual void	Show(void);
	virtual void	Hide(void);
	
									/** Size and Location **/
	virtual void	Place( long hEncl, long vEncl, Boolean redraw);
	virtual void	Offset( long hOffset, long vOffset, Boolean redraw);
	virtual void	ChangeSize(Rect *delta, Boolean redraw);
	virtual void	AdjustToEnclosure(Rect *deltaEncl);
	virtual void	AdjustHoriz(Rect *deltaEncl, Rect *delta, short *offset,
						Boolean *moved, Boolean *sized);
	virtual void	AdjustVert(Rect *deltaEncl, Rect *delta, short *offset,
						Boolean *moved, Boolean *sized);
	virtual void	EnclosureScrolled( long hOffset, long vOffset);
									
									/** Adapting **/
	virtual void	FitToEnclosure(Boolean horizFit, Boolean vertFit);
	virtual void	FitToEnclFrame(Boolean horizFit, Boolean vertFit);
	virtual void	CenterWithinEnclosure(Boolean horizCenter, Boolean vertCenter);

									/** Drawing **/
	virtual void	Draw(Rect *area);
	virtual void	DrawAll(Rect *area);
	virtual void	Refresh(void);
	virtual void	RefreshRect( Rect *area);
	virtual void 	RefreshLongRect( LongRect *area);
	virtual void	RefreshBorder( void);
						
									/** Printing **/
					
	virtual void	Paginate( CPrinter *aPrinter, short pageWidth, short pageHeight);				
	virtual void	AboutToPrint(short *firstPage, short *lastPage);
	virtual void	PrintPage(short pageNum, short pageWidth, short pageHeight, CPrinter *aPrinter);
	virtual void	DonePrinting(void);
	virtual void	PrepareToPrint(void);
	
									/** Calibrating **/
	virtual void	Prepare(void);
	virtual void	RestoreEnvironment(void);
	virtual void	CalcFrame(void);
	virtual void	ResizeFrame(Rect *delta);
	virtual void	CalcAperture(void);
	
	virtual void	TrackMouse(CMouseTask *theTask, LongPt *startPt, LongRect *pinRect);

									/** Coordinate Transforms **/
									
	virtual void	WindToFrame( Point windPt, LongPt *framePt);
	virtual void	WindToFrameR( Rect *windRect, LongRect *frameRect);
	virtual void	FrameToWind( LongPt *framePt, Point *windPt);
	virtual void	FrameToWindR( LongRect *frameRect, Rect *windRect);
	virtual void	EnclToFrame( LongPt *thePoint);
	virtual void	EnclToFrameR( LongRect *theRect);
	virtual void	FrameToEncl( LongPt *thePoint);
	virtual void	FrameToEnclR( LongRect *theRect);
	virtual void	FrameToGlobalR( LongRect *frameRect, Rect *globalRect);
	
	virtual void	QDToFrame( Point qdPt, LongPt *framePt);
	virtual void	QDToFrameR( Rect *qdRect, LongRect *frameRect);
	virtual void	FrameToQD( LongPt *framePt, Point *qdPt);
	virtual void	FrameToQDR( LongRect *frameRect, Rect *qdRect);
	
	virtual Boolean SectAperture( LongRect *srcRect, Rect *destRect);

	
};

	// TRUE/FALSE synonyms for some common Boolean parameters
enum
{
	kNoRedraw = FALSE,
	kRedraw,
	kNotHorizontal = FALSE,
	kDoHorizontal,
	kNotVertical = FALSE,
	kDoVertical
};


typedef struct	{						/* Pane template					*/
	ViewTemp		sViewTemp;
	short			width;
	short			height;
	short			hEncl;
	short			vEncl;
	short			hSizing;
	short			vSizing;
	short			autoRefresh;
	short			printClip;
} PaneTemp, *PaneTempP;

		/* Functions passed as arguments to list methods */

	void	Pane_Draw(CPane *thePane, Rect *area);
	void	Pane_AdjustToEnclosure(CPane *thePane, Rect *delta);