/******************************************************************************
 CView.h

		Interface for the View Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.

	TCL 1.1 CHANGES
	[
		- added GetWantsClicks method
		- added canBeGopher instance variable, and SetCanBeGopher and
		  CanBeGopher methods for accessing
		- added ID instance variable, GetID, SetID,
		  and FindByID methods
		- added MatchView method and typedef for MatchViewProc
		- added ForcePrepare method
		- for 32-bit view support, changed GetFrame, GetInterior, GetAperture,
		  and FrameToGlobalR to use LongRects.
		- added usingLongCoord instance variable and UseLongCoordinates
		  method.
		- added ForceNextPrepare static method
		- moved TrackMouse to CPane. It needs access to some coordinate
		  transformations provided by CPane.
		- added helpResIndex instance variable, and GetBalloonInfo, ShowHelpBalloon, 
		  and GetHelpResID methods
		- added cPreparedView, cCurrHelpView, cLastHelpView,
		  class variables
	]
 ******************************************************************************/
 
#define _H_CView

#include "CBureaucrat.h"				/* Interface for its superclass		*/
#include "LongCoordinates.h"


	// forward declarations
	
class CView;
class CList;
struct HMMessageRecord;

typedef Boolean (*MatchViewProc)( CView*, void*);

class CView : public CBureaucrat {		/* Class Declaration				*/

public:

	GrafPtr		macPort;				/* Mac drawing port for the image	*/
	CView		*itsEnclosure;			/* Enclosing view					*/
	CList		*itsSubviews;			/* Views within this view			*/
	Boolean		visible;				/* Is the view visible?				*/
	Boolean		active;					/* Is the view active?				*/
	Boolean		wantsClicks;			/* Does view handle mouse clicks?	*/
	Boolean		canBeGopher;			/* Can this view become the gopher? */
	long		ID;						/* identifier for this view			*/
	Boolean		usingLongCoord;			/* TRUE if using 32-bit coordinates	*/
	short 		helpResIndex;			/* ballon help info for the view	*/

	static		CView *cPreparedView;	/* currently Prepared view			*/
	static		CView *cCurrHelpView;	/* used for Balloon help			*/
	static 		CView *cLastHelpView;	/* also for Balloon help			*/
										
								/** Instance Methods **/
									/** Construction/Destruction **/
	void		IView(CView *anEnclosure, CBureaucrat *aSupervisor);
	virtual void		IViewRes(ResType rType, short resID,
							CView *anEnclosure, CBureaucrat *aSupervisor);
	virtual void		IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
							Ptr viewData);
	virtual void		Dispose(void);

									/** Accessing **/
	virtual Boolean		IsVisible(void);
	virtual Boolean		IsActive(void);
	virtual Boolean		ReallyVisible(void);
	virtual GrafPtr		GetMacPort(void);
	virtual void		GetOrigin(long *theHOrigin, long *theVOrigin);
	virtual void		GetFrame(LongRect *theFrame);
	virtual void		GetInterior(LongRect *theInterior);
	virtual void		GetAperture(LongRect *theAperture);
	virtual Boolean		Contains(Point thePoint);
	virtual void		SetWantsClicks(Boolean aWantsClicks);
	virtual Boolean		GetWantsClicks( void);
	virtual void		SetCanBeGopher( Boolean fCanBeGopher);
	virtual Boolean		CanBeGopher( void);
	virtual void		SetID( long anIdentifier);
	virtual long		GetID( void);
	virtual void		UseLongCoordinates( Boolean fUsing);

									/** Appearance **/
	virtual void		Show(void);
	virtual void		Hide(void);
	virtual void		Activate(void);
	virtual void		Deactivate(void);
	
									/** Mouse **/
	virtual void		DispatchClick(EventRecord *macEvent);
	virtual void		DoClick(Point hitPt, short modifierKeys, long when);
	virtual Boolean		HitSamePart(Point pointA, Point pointB);
	virtual void		DoMouseUp(EventRecord *macEvent);
	
									/** Cursor **/
	virtual void		DispatchCursor(Point where, RgnHandle mouseRgn);
	virtual void		AdjustCursor(Point where, RgnHandle mouseRgn);
	
	virtual void		GetBalloonInfo( struct HMMessageRecord *helpData, 
	                           Point *tip,
	                           RectPtr alternateRect,
	                           Ptr *tipProc,
	                           short *theProc,
	                           short *variant,
	                           short *method);
	                           
	virtual void		ShowHelpBalloon( struct HMMessageRecord *helpData, 
	                           Point tip,
	                           RectPtr alternateRect,
	                           Ptr tipProc,
	                           short theProc,
	                           short variant,
	                           short method);
	
	virtual short		GetHelpResID( void);

									/** Subview Management **/
	virtual void		AddSubview(CView *theSubview);
	virtual void		RemoveSubview(CView *theSubview);
	virtual CView*		FindSubview(Point hitPt);
	virtual void		SubpaneLocation(long hEncl, long vEncl,
								long *hLocation, long *vLocation);
	virtual CView*		FindViewByID( long anID);
	virtual CView*		MatchView( MatchViewProc matchProc, void* matchData);
	virtual void		Prepare(void);
	virtual void		FrameToGlobalR( LongRect *frameRect, Rect *globalRect);
	
	static void	ForceNextPrepare( void);

};


typedef struct	{						/* View template					*/
	short		visible;
	short		active;
	short		wantsClicks;
} ViewTemp, *ViewTempP;


void	CountClicks(CView *hitView, EventRecord *macEvent);