/******************************************************************************
 CWindow.h

		Interface for the Window Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- added isModal instance variable and IsModal method.
		- added INewWindow method to support creation without WIND resource
		- added procID instance variable to remember the defproc ID used
		  to create the window.
		- added isColor instance variable and IsColor method
		- added helpResID instance variable, SetHelpResID, GetHelpResID
		  methods
	]

 ******************************************************************************/
 
#define _H_CWindow

#include "CView.h"						/* Interface for its superclass		*/

class CDesktop;
class CDirector;

class CWindow : public CView {			/* Class Declaration				*/

public:
								/** Instance Variables **/
	short		procID;					/* defproc ID used to create the window */
	Rect		sizeRect;				/* Max and Min size for the window	*/
	Boolean		floating;				/* Is this a floating window?		*/
	Boolean		isColor;				/* Is this a color window?			*/
	Boolean		isModal;				/* Is it currently modal?			*/
	Boolean		actClick;				/* Does it process a mouse click	*/
										/*   which activates the window?	*/
	Point		location;				/* Location of window, used when	*/
										/*   "hiding" while suspended		*/
	short		helpResID;				/* index of 'hrct' resource for		*/
										/* help balloons					*/
	
								/** Instance Methods **/
									/** Construction/Desctruction **/
	void		IWindow(short WINDid, Boolean aFloating,
						CDesktop *anEnclosure, CDirector *aSupervisor);
	void		INewWindow( Rect *bounds, Boolean fVisible, short procID,
						Boolean fFloating, Boolean fHasGoAway,
						CDesktop *anEnclosure, CDirector *aSupervisor);
	void		IWindowX( void);
	virtual void		Dispose(void);
	virtual void		MakeMacWindow(short WINDid);
	virtual void		MakeNewMacWindow( Rect *bounds, short aProcID, Boolean fHasGoAway);
	virtual void		Close(void);

									/** Accessing **/
	virtual void		GetFrame( LongRect *theFrame);
	virtual void		GetInterior( LongRect *theInterior);
	virtual void		GetAperture( LongRect *theAperture);
	virtual Boolean		IsFloating(void);
	virtual Boolean		IsModal( void);
	virtual Boolean		IsColor( void);
	virtual void		SetModal( Boolean fModal);
	virtual void		SetTitle(Str255 theTitle);
	virtual void		GetTitle(Str255 theTitle);
	virtual void		SetActClick(Boolean anActClick);
	virtual Boolean		WantsActClick(void);
	virtual Boolean		Contains(Point thePoint);
	virtual void		SetSizeRect(Rect *aSizeRect);
	virtual void		SetStdState(Rect *aStdState);
	virtual void		SetHelpResID( short aResID);
	virtual short		GetHelpResID( void);

									/** Appearance **/
	virtual void		Show(void);
	virtual void		Hide(void);
	virtual void		Activate(void);
	virtual void		Deactivate(void);
	virtual void		Select(void);
	virtual void		ShowResume(void);
	virtual void		HideSuspend(void);
	virtual void		ShowOrHide(Boolean showFlag);
	
									/** Size and Location **/
	virtual void		Drag(EventRecord *macEvent);
	virtual void		Resize(EventRecord *macEvent);
	virtual void		Zoom(short direction);
	virtual void		Move(short hGlobal, short vGlobal);
	virtual void		ChangeSize(short width, short height);
	virtual void		MoveOffScreen(void);
	
									/** Drawing **/
	virtual void		Update(void);
	virtual void		Prepare(void);
	
									/** Mouse **/
	virtual void		DispatchClick(EventRecord *macEvent);
	virtual void		DispatchCursor(Point where, RgnHandle mouseRgn);

	virtual void		FrameToGlobalR(LongRect *frameRect, Rect *globalRect);
};


	// symbol constants for Boolean parameters
enum
{
	kModeless = FALSE,
	kModal,
	kNotVisible = FALSE,
	kVisible,
	kNoGoAway = FALSE,
	kHasGoAway,
	kNotFloating = FALSE,
	kFloating
};