/******************************************************************************
 CScrollBar.c

							The ScrollBar Class
		
		A standard Mac scroll bar control.
	
	SUPERCLASS = CControl
	
	Copyright  1989 Symantec Corporation. All rights reserved.

	TCL 1.1 CHANGES
	[
		- IScrollBar now respects the visible flag in the
		  CNTL resource.
	]

 ******************************************************************************/

#include "Global.h"
#include "Constants.h"
#include "CScrollBar.h"
#include "CScrollPane.h"

/******************************************************************************
 IScrollBar

		Initialize a ScrollBar object
 ******************************************************************************/

void	CScrollBar::IScrollBar(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	Orientation		anOrientation,
	short			aLength,
	short			aHEncl,
	short			aVEncl)
{
	Point			enclOrigin;
    CNTLtemplateHndl cntl;
    short			 savedVis;          

										/* Initialize View. Note that there	*/
										/*   is no IControl() method.		*/
	CView::IView(anEnclosure, aSupervisor);
	
	itsBorder = NULL;
	
	macPort = itsEnclosure->GetMacPort();
	SetPort( macPort);
	SetOrigin(0, 0);
	ForceNextPrepare();
	
										/* Use a resource template			*/
	
	cntl = (CNTLtemplateHndl) GetResource( 'CNTL', CNTL_SBAR);
	FailNILRes( cntl);
	savedVis = (**cntl).visible;
	(**cntl).visible = 0;

	macControl = GetNewControl(CNTL_SBAR, macPort);
	FailNIL(macControl);

	visible = FALSE;
	active = TRUE;
	wantsClicks = TRUE;
	autoRefresh = TRUE;
	
	hEncl = aHEncl;
	vEncl = aVEncl;
		
	theOrientation = anOrientation;
	theThumbFunc = NULL;
										/* Dimensions and sizing options	*/
										/*   are based on the orientation	*/
	if (theOrientation == HORIZONTAL) {
		width = aLength;
		height = SBARSIZE;
		hSizing = sizELASTIC;
		vSizing = sizFIXEDBOTTOM;
	} else {
		width = SBARSIZE;
		height = aLength;
		hSizing = sizFIXEDRIGHT;
		vSizing = sizELASTIC;
	}
	
    printClip = clipFRAME;
    printing = FALSE;
    
	CalcFrame();						/* Set up Frame coordinates			*/
	CalcAperture();						/* Determine visible portion		*/
	
	
	itsEnvironment = NULL;
	itsEnclosure->AddSubview(this);		/* This is a subview of enclosure	*/
	SetCRefCon(macControl, (long) this);

	MoveControl(macControl, -(short) hOrigin, -(short) vOrigin);
	if (savedVis) 
	{
		Show();
		cntl = (CNTLtemplateHndl) GetResource( 'CNTL', CNTL_SBAR);
		FailNILRes( cntl);
		(**cntl).visible = savedVis;
	}

	Prepare();
	SizeControl(macControl, width, height);
	Refresh();
}
		

/******************************************************************************
 Draw {OVERRIDE}

		Draw a scroll bar
 ******************************************************************************/

void	CScrollBar::Draw(
	Rect		*area)
{
	Rect			outline;			/* Outline of a scroll bar			*/
	
	if (IsActive()) {
		if ((**macControl).contrlVis != 255) {
										/* Scroll bar is not currently		*/
										/*   visible so show it				*/
			MoveControl(macControl, -(short) hOrigin, -(short) vOrigin);
			ShowControl(macControl);
		} else {						/* Just draw the scroll bar			*/
			Draw1Control(macControl);
		}
	} else {							/* Don't draw scroll bar at all if	*/
										/*   it is inactive. Just draw an	*/
										/*   empty box around where it is.	*/
		FrameToWindR( &frame, &outline);
		PenNormal();
		FrameRect(&outline);
		InsetRect(&outline, 1, 1);
		EraseRect(&outline);
	}
}
	
		
/******************************************************************************
 Activate {OVERRIDE}

		Activate a scroll bar
 ******************************************************************************/

void	CScrollBar::Activate()
{
	Rect	tempRect;
	
	if (!active) {
		FrameToWindR( &frame, &tempRect);
		Prepare();
										/* Force redraw of entire scroll	*/
		InsetRect(&tempRect, 1, 1);		/*   bar, excluding the one pixel	*/
		InvalRect(&tempRect);			/*   outline						*/
		active = TRUE;
		ForceNextPrepare();
	}
}
	

/******************************************************************************
 Deactivate {OVERRIDE}

		Deactivate a Scroll Bar. An inactive scroll bar is hidden -- only
		its outline is drawn.
 ******************************************************************************/

void	CScrollBar::Deactivate()
{
	Rect	tempRect;
	
	if (active) {
		active = FALSE;
		ForceNextPrepare();
		(**macControl).contrlVis = 0;	/* Stuff value directly into		*/
										/*   Control Record instead of		*/
										/*   using traps to prevent			*/
										/*   automatic redraw. This is OK	*/
										/*   since we check this value in	*/
										/*   our Draw() method and never	*/
										/*   use the DrawControls trap.		*/

		FrameToWindR( &aperture, &tempRect);
										/* Force our own drawing of an		*/
		DrawAll(&tempRect);				/*   inactive scroll bar			*/
	}
}
	

/******************************************************************************
 DoClick {OVERRIDE}

		Handle a click inside a scroll bar. Let the inherited method to
		do all the work, then tack on .
 ******************************************************************************/

void	CScrollBar::DoClick(
	Point		hitPt,
	short		modifierKeys,
	long		when)
{
	inherited::DoClick(hitPt, modifierKeys, when);
	
										/* Act of scrolling may require		*/
										/*   adjustment of a scroll bar		*/
	if (member(itsEnclosure, CScrollPane)) {
		((CScrollPane*)itsEnclosure)->AdjustScrollMax();
	}
}
		

/******************************************************************************
 DoThumbDragged {OVERRIDE}

		User dragged thumb of the scroll bar. Call function to respond
		appropriately.
 ******************************************************************************/

void	CScrollBar::DoThumbDragged(
	short		delta)					/* Change in control setting		*/
{
	if (theThumbFunc)					/* Call thumb function if one		*/
		(*theThumbFunc)(this, delta);	/*   exists; otherwise do nothing	*/
}		


/******************************************************************************
 SetThumbFunc

		Set the pointer to the function to be called after the thumb of
		the scroll bar is dragged
 ******************************************************************************/

void	CScrollBar::SetThumbFunc(
	VoidFunc	aThumbFunc)				/* Function pointer					*/
{
	theThumbFunc = aThumbFunc;			/* Set instance variable			*/
}