/******************************************************************************
 CPictFile.c

							The PictFile Class
		
	Handles reading and writing PICT files.
	
	SUPERCLASS = CDataFile
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CPictFile.h"
#include "CApplication.h"
#include "TCLUtilities.h"

/**** Global Variables ****/

extern CApplication	*gApplication;		/* Application object				*/

/*** Class Constants ***/

#define		PICT_HEADER_SIZE	512		/* Bytes in head of a PICT file		*/


/******************************************************************************
 IPictFile

		Initialize a PictFile
 ******************************************************************************/

void	CPictFile::IPictFile()
{
    CDataFile::IDataFile();
	
	header = NewHandleCanFail((long)PICT_HEADER_SIZE);
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a PictFile
 ******************************************************************************/

void	CPictFile::Dispose()
{
	ForgetHandle( header);

	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 ReadAll {OVERRIDE}

		Read contents of a PICT file into memory
 ******************************************************************************/

Handle	CPictFile::ReadAll( void)
{
	long		length;					/* Length of file in bytes			*/
	Handle		h = NULL;

	TRY
	{
		SetMark( 0L, fsFromStart);
		length = PICT_HEADER_SIZE;
		HLock(header);
		ReadSome( *header, length);
		HUnlock(header);
		
		length = GetLength() - PICT_HEADER_SIZE;
		
		h = NewHandleCanFail(length);
		FailNIL( h);
		
		SetMark((long)PICT_HEADER_SIZE, fsFromStart);
		HLock( h);
		ReadSome( *h, length);
		HUnlock( h);
	}
	CATCH
	{
		HUnlock( header);
		ForgetHandle( h);
	}
	ENDTRY;
	
	return h;
}


/******************************************************************************
 WriteAll {OVERRIDE}

		Write out a file. Contents will be a PicHandle. We must write out
		both the fixed header and the contents.
 ******************************************************************************/

void	CPictFile::WriteAll(
	Handle		contents)
{
	long		length;
	
	TRY
	{
		SetMark( 0L, fsFromStart);
		HLock(header);
		WriteSome( *header, (long)PICT_HEADER_SIZE);
		HUnlock( header);
	
		SetMark( (long)PICT_HEADER_SIZE, fsFromStart);
		length = GetHandleSize(contents);
		HLock(contents);
		WriteSome(*contents, length);
		HUnlock(contents);
		
		SetLength(length + PICT_HEADER_SIZE);
	}
	CATCH
	{
		HUnlock(header);
		HUnlock( contents);
	}
	ENDTRY;
}
