/******************************************************************************
 LongQD.c

		Wrapper functions for some common QuickDraw functions. These
		wrappers accept long coordinates. They assume the coordinates
		are in the Frame coordinates of the currently prepared view.
		
		Note that, when possible, it would be much more convenient
		to convert from Frame to QuickDraw coordinates once, than it
		is to repeatedly call these wrapper functions.
		
		Also remember that although these functions take long coordinate
		structures, it is not possible to draw primitives larger than
		the QuickDraw coordinate space, i.e. you can't draw a 100,000 X
		100,000 pixel rectangle directly.
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "LongQD.h"
#include "CPane.h"


static void ConvertRect( LongRect *lr, Rect *r)
{
	ASSERT(CView::cPreparedView != NULL);
	ASSERT( member( CView::cPreparedView, CPane));
	
	((CPane*)CView::cPreparedView)->FrameToQDR( lr, r);
}

static void ConvertPt( LongPt *lPt, Point *pt)
{
	ASSERT(CView::cPreparedView != NULL);
	ASSERT( member( CView::cPreparedView, CPane));
	
	((CPane*)CView::cPreparedView)->FrameToQD( lPt, pt);
}

void LEraseRect( LongRect	*area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	EraseRect( &r);
}

void LFrameRect( LongRect *area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	FrameRect( &r);
}

void LInvertRect( LongRect *area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	InvertRect( &r);
}

void LPaintRect( LongRect *area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	PaintRect( &r);
}

void LFillRect( LongRect *area, Pattern pat)
{
	Rect	r;
	
	ConvertRect( area, &r);
	FillRect( &r, pat);
}

void LEraseOval( LongRect	*area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	EraseOval( &r);
}

void LFrameOval( LongRect *area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	FrameOval( &r);
}

void LInvertOval( LongRect *area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	InvertOval( &r);
}

void LPaintOval( LongRect *area)
{
	Rect	r;
	
	ConvertRect( area, &r);
	PaintOval( &r);
}

void LFillOval( LongRect *area, Pattern pat)
{
	Rect	r;
	
	ConvertRect( area, &r);
	FillOval( &r, pat);
}

void LEraseRoundRect( LongRect	*area, short ovWidth, short ovHeight)
{
	Rect	r;
	
	ConvertRect( area, &r);
	EraseRoundRect( &r, ovWidth, ovHeight);
	
}

void LFrameRoundRect( LongRect *area, short ovWidth, short ovHeight)
{
	Rect	r;
	
	ConvertRect( area, &r);
	FrameRoundRect( &r, ovWidth, ovHeight);
}

void LInvertRoundRect( LongRect *area, short ovWidth, short ovHeight)
{
	Rect	r;
	
	ConvertRect( area, &r);
	InvertRoundRect( &r, ovWidth, ovHeight);
}

void LPaintRoundRect( LongRect *area, short ovWidth, short ovHeight)
{
	Rect	r;
	
	ConvertRect( area, &r);
	PaintRoundRect( &r, ovWidth, ovHeight);
}

void LFillRoundRect( LongRect *area, short ovWidth, short ovHeight, Pattern pat)
{
	Rect	r;
	
	ConvertRect( area, &r);
	FillRoundRect( &r, ovWidth, ovHeight, pat);
}

void LCopyBits( BitMap* srcBits, BitMap* dstBits, 
				LongRect *srcRect, LongRect *dstRect, 
				short mode,RgnHandle mask)
{
	Rect src, dest;
	
	ConvertRect( srcRect, &src);
	ConvertRect( dstRect, &dest);
	CopyBits( srcBits, dstBits, &src, &dest, mode, mask);
	
}

void LMoveTo( long hLoc, long vLoc)
{
	LongPt	lPt;
	Point	qdPt;
	
	
	lPt.h = hLoc;
	lPt.v = vLoc;
	ConvertPt( &lPt, &qdPt);
	MoveTo( qdPt.h, qdPt.v);

}

void LLineTo( long hLoc, long vLoc)
{
	LongPt	lPt;
	Point	qdPt;
	
	
	lPt.h = hLoc;
	lPt.v = vLoc;
	ConvertPt( &lPt, &qdPt);
	LineTo( qdPt.h, qdPt.v);
}

void LRectRgn( RgnHandle rgn, LongRect *rect)
{
	Rect r;
	
	ConvertRect( rect, &r);
	RectRgn( rgn, &r);
}

void LClipRect( LongRect *rect)
{
	Rect r;
	
	ConvertRect( rect, &r);
	ClipRect( &r);
}