/******************************************************************************
 CStyleTEStyleTask.c

		A subclass of CTextStyleTask that handles undo of formatting commands
		for a CStyleText pane. An instance of this class is automatically
		created for you when you use the CStyleText class.
		
	SUPERCLASS = CTextStyleTask
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CStyleTEStyleTask.h"
#include "Commands.h"
#include "Global.h"

/******************************************************************************
 IStyleTEStyleTask
 
 	Initialize a CStyleTEStyleTask. aStyleCmd should be the formatting command 
 	that the task is to respond to. FirstTaskIndex is  the index into the strUNDO 
 	'STR#' where the text undo labels are located. These are typically "Typing", 
 	"Cut", "Copy", "Paste", "Clear", and "Formatting". 
******************************************************************************/

void CStyleTEStyleTask::IStyleTEStyleTask( CStyleText *aTextPane, 
				long aStyleCmd, short firstTaskIndex)
{
	long selStart, selEnd;
	
	oldStyles = NULL;
	macTE = aTextPane->macTE;
	
	CTextStyleTask::ITextStyleTask( aTextPane, aStyleCmd, firstTaskIndex);
	
	itsTextPane->GetSelection( &selStart, &selEnd);
	this->selStart = selStart;
	this->selEnd = selEnd;
	
}	/* CStyleTEStyleTask::IStyleTEStyleTask */

/******************************************************************************
 Undo
 
 	Undo the previously applied formatting. This method swaps the current and
 	previous styles, so it can perform both undo and redo.
******************************************************************************/

void CStyleTEStyleTask::Undo( void)
{
	StScrpHandle	swapStyles, newStyles;
	CStyleText		*styleText = (CStyleText*) itsTextPane;
	
	swapStyles = newStyles = NULL;

	TRY
	{
		if (styleAttribute < doAlign)
		{
			
			itsTextPane->Prepare();	
			itsTextPane->SetSelection( selStart, selEnd, TRUE);
			
			swapStyles = oldStyles;		/* save the current styling		*/
			oldStyles = NULL;
			
			SaveStyle();
		
			newStyles = oldStyles;
			oldStyles = swapStyles;		/* restore the previous styling */
			swapStyles = NULL;
		
			RestoreStyle();
			
			ForgetHandle( oldStyles);
			
			oldStyles = newStyles;		/* swap current and previous	*/
			
			styleText->SetSpacingCmd( styleText->GetSpacingCmd());
			styleText->SetWholeLines( styleText->GetWholeLines());
			styleText->AdjustBounds();
			
			undone = !undone;

		}
		else inherited::Undo();
	}
	CATCH
	{
		ForgetHandle( oldStyles);
		ForgetHandle( newStyles);
		ForgetHandle( swapStyles);
	}
	ENDTRY;

}	/* CStyleTEStyleTask::Undo */

/******************************************************************************
 SaveStyle
 
 	Saves the current styling into oldStyles.
******************************************************************************/

void CStyleTEStyleTask::SaveStyle( void)
{
	oldStyles = NULL;	/* GetStyleScrap may fail	*/
	
	oldStyles = ((CStyleText*)itsTextPane)->GetStyleScrap();

}	/* CStyleTEStyleTask::SaveStyle */

/******************************************************************************
 RestoreStyle
 
 	Restores the saved styles from oldStyles back to the text.
******************************************************************************/

void CStyleTEStyleTask::RestoreStyle( void)
{
	if (styleAttribute < doAlign)
	{	
		/* +++ work around problem with redoing cmdPlain on styled text? */
		if ((styleCmd == cmdPlain) && (undone == TRUE))
			itsTextPane->SetFontStyle( NOTHING);
		else
			((CStyleText*)itsTextPane)->SetStyleScrap( selStart, selEnd, oldStyles, TRUE);		
	}
	else
		inherited::RestoreStyle();
		
}	/* CStyleTEStyleTask::RestoreStyle */

/******************************************************************************
 Dispose
******************************************************************************/

void CStyleTEStyleTask::Dispose( void)
{
	ForgetHandle( oldStyles);
	inherited::Dispose();

}	/* CStyleTEStyleTask::Dispose */